<?php

/**
 * @file
 * Contains \Drupal\miniorange_ldap\Form\MiniorangeLDAP.
 */

namespace Drupal\ldap_auth\Form;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\FormBase;
use Drupal\ldap_auth\handler;
use Drupal\ldap_auth\LDAPLOGGERS;
use Drupal\ldap_auth\LDAPFlow;
use Drupal\ldap_auth\MiniorangeLdapSupport;
use Drupal\ldap_auth\Utilities;
use Drupal\Component\Utility\Html;

class MiniorangeLDAP extends FormBase{
  public function getFormId() {
    return 'miniorange_ldap_config_client';
  }
  public function buildForm(array $form, FormStateInterface $form_state){
      global $base_url;
    $ldap_connect = new LDAPFlow();
    \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_disabled', FALSE)->save();


    $form['markup_library'] = array(
        '#attached' => array(
            'library' => array(
                "ldap_auth/ldap_auth.admin",
                "ldap_auth/ldap_auth.main",
            )
        ),
    );

    if(!Utilities::isLDAPInstalled()){
        $form['markup_reg_msg'] = array(
            '#markup' => '<div class="mo_ldap_enable_extension_message"><b>The PHP LDAP extension is not enabled.</b><br> Please Enable the PHP LDAP Extension for you server to continue. If you want, you refer to the steps given on the link  <a target="_blank" href="https://faq.miniorange.com/knowledgebase/how-to-enable-php-ldap-extension/" >here</a> to enable the extension for your server.</div><br>',
        );
    }
    $next_disabled = true;
    if(\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_test_conn_enabled') == 1 )
      $next_disabled = false;
    $status= \Drupal::config('ldap_auth.settings')->get('miniorange_ldap_config_status');
    if($status=='')
      $status = 'two';

    $config_step = \Drupal::config('ldap_auth.settings')->get('miniorange_ldap_steps');
    switch ($config_step) {
      case 0:
        $navbar_val = 1;
    break;
      case 1:
        $navbar_val = 20;
    break;
      case 2:
        $navbar_val = 44;
    break;
      case 3:
        $navbar_val = 68;
        break;
      case 4:
        $navbar_val = 100;
        break;
      default:
        $navbar_val = 1;
    }
    if($navbar_val == 100 ){
      $form['#prefix'] = '
    <div class="mo_ldap_table_layout_1">
        <div class="mo_ldap_table_layout">
            <strong>Note: </strong>You can get the below information from your LDAP administrator</strong><div class="btn-right"><a class="button button--primary" href ="https://www.youtube.com/watch?v=wBe8T6FLKx4" target="_blank">Setup Video</a> <a class="button button--primary" href="https://plugins.miniorange.com/guide-to-configure-ldap-ad-integration-module-for-drupal" target="_blank">Setup Guide</a><br></div>
        <br><br>';
    }else {
      $form['#prefix'] = '
    <div class="mo_ldap_table_layout_1">
        <div class="container_m1">
          <div class="table_navbar"><table><th>Step 1: <br>Contact LDAP Server</th><th>Step 2: <br>Perform Test Connection</th><th>Step 3: <br>Select Search Base/Filter</th><th>Step 4: <br>Enable Login using LDAP</th><th>All Done</th></table></div>
          <progress id="determinate"  value="' . $navbar_val . '" min="0" max="100"> 25% </progress>
        </div>
            <br>
        <div class="mo_ldap_table_layout">
            <strong>Note: </strong>You can get the below information from your LDAP administrator</strong><div class="btn-right"><a class="button button--primary" href ="https://www.youtube.com/watch?v=wBe8T6FLKx4" target="_blank">Setup Video</a> <a class="button button--primary" href="https://plugins.miniorange.com/guide-to-configure-ldap-ad-integration-module-for-drupal" target="_blank">Setup Guide</a><br></div>
        <br><br>';
    }
    if($status=='review_config'){
        $form['miniorange_ldap_enable_ldap_markup'] = array(
        '#markup' => "<h1><b>Login With LDAP</b></h1><hr><br>",
        );
        $form['miniorange_ldap_enable_ldap'] = array(
            '#type' => 'checkbox',
            '#title' => t('Enable Login with LDAP '),
            '#default_value' => \Drupal::config('ldap_auth.settings')->get('miniorange_ldap_enable_ldap'),
        );
        $form['miniorange_ldap_enable_auto_reg'] = array(
          '#type' => 'checkbox',
          '#disabled' => 'true',
          '#title' => t('Enable Auto Creation of users if they do not exist in Drupal <a href="' . $base_url .'/admin/config/people/ldap_auth/Licensing"><strong>[Premium, All-inclusive]</strong></a>'),
          '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_enable_auto_reg'),
        );
        $form['set_of_radiobuttons']['miniorange_ldap_authentication'] = array(
            '#type' => 'radios',
            '#disabled' => true,
            '#title' => t('Authentication restrictions: <a href="' . $base_url .'/admin/config/people/ldap_auth/Licensing">[Premium, All-inclusive]</a>'),
            '#description' => t('Only particular personalities will be able to login by selecting the above option.'),
            '#tree' => TRUE,
            '#default_value' => is_null(\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_authentication'))?1:\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_authentication'),
            '#options' => array(0 => t('User can login using both their Drupal or LDAP credentials'), 1 => t('User can login in Drupal using their LDAP credentials and Drupal admins can also login using their local Drupal credentials'), 2 => t('Users can only login using their LDAP credentials')),
        );
        $form['ldap_server'] = array(
            '#markup' => "<br><br>
                <h1><b>LDAP Connection Information</b></h1><hr><br><br><h3>LDAP Server:</h3>",
        );
        $form['ldap_server_url_markup_start'] = array(
            '#markup' => '<div class="ldap_Server_row">',
        );

        $form['miniorange_ldap_options'] = array(
            '#type' => 'value',
            '#id' => 'miniorange_ldap_options',
            '#value' => array(
              'ldap://' => t('ldap://'),
              'ldaps://' => t('ldaps://'),),
        );

        $form['miniorange_ldap_protocol'] = array(
        '#id' => 'miniorange_ldap_protocol',
        '#type' => 'select',
        '#prefix' => '<div class="ldap-column left">',
        '#suffix' => '</div>',
        '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_protocol'),
        '#options' => $form['miniorange_ldap_options']['#value'],
        '#attributes' => array('style'=>'width:100%'),
        );

        $form['miniorange_ldap_server_address'] = array(
            '#type' => 'textfield',
            '#id' => 'miniorange_ldap_server_address',
            '#prefix' => '<div class="ldap-column middle">',
            '#suffix' => '</div>',
            '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_server_address'),
            '#attributes' => array('style' => 'width:100%;','placeholder' => 'Enter your server-address or IP'),
        );
        $form['miniorange_ldap_server_port_number'] = array(
            '#type' => 'textfield',
            '#prefix' => '<div class="ldap-column right">',
            '#suffix' => '</div>',
            '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_server_port_number'),
            '#attributes' => array('style' => 'width:100%;','placeholder' => '<port>'),
        );
        $form['ldap_server_url_markup_end'] = array(
            '#markup' => '</div><p><i>Select ldap or ldaps from the above dropdown list. Specify the host name for the LDAP server in the above text field. Edit the port number if you have custom port number.</i><br><br>
                            LDAPS allows you to secure your LDAP server connection between client and server application to encrypt the communication.
                            </p>',
        );

        $form['miniorange_ldap_contact_server_button'] = array(
            '#type' => 'submit',
            '#value' => t('Contact LDAP Server'),
            '#submit' => array('::test_ldap_connection'),
        );
        $form['miniorange_ldap_enable_tls'] = array(
            '#prefix' => '<br>',
            '#type' => 'checkbox',
            '#disabled' => true,
            '#id' => 'check',
            '#title' => t('Enable TLS (Check this only if your server use TLS Connection)  <a href="' . $base_url .'/admin/config/people/ldap_auth/Licensing"><strong>[Premium, All-inclusive]</strong></a>'),
            '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_enable_tls'),
        );
        global $base_url;
        $form['miniorange_ldap_anonymous_bind_markup_2'] = array(
            '#markup' => '<br><div class="mo_ldap_highlight_background_note_1" >In case you do not have any authentication setup and wish to perform anonymous bind to your server, please click on the Next button and continue with your setup.</div>',
        );
        $form['miniorange_ldap_server_account_username'] = array(
            '#type' => 'textfield',
            '#title' => t('Service Account Username:'),
            '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_server_account_username'),
            '#description' => "This service account username will be used to establish the connection. Specify the Service Account Username of the LDAP server in the either way as follows Username@domainname or domainname\Username. or Distinguished Name(DN) format",
            '#attributes' => array('style' => 'width:73%;','placeholder' => 'CN=service,DC=domain,DC=com'),
        );
        $form['miniorange_ldap_server_account_password'] = array(
            '#type' => 'password',
            '#title' => t('Service Account Password:'),
            '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_server_account_password'),
            '#attributes' => array('style' => 'width:73%;','placeholder' => 'Enter password for Service Account'),
        );
        $form['miniorange_ldap_test_connection_button'] = array(
            '#type' => 'submit',
            '#disabled' => $next_disabled,
            '#prefix' => '<br>',
            '#value' => t('Test Connection'),
            '#submit' => array('::test_connection_ldap'),
        );
        $form['troubleshooting_1'] = array(
            '#markup' => "
                <div style='background-color: white; padding: 10px ;margin-left: 20px; width: 70%' id='enable_ldap'>
                    <input type='button' style='background-color: #008CBA;border: none;color: white;padding: 8px 20px;text-align: center;text-decoration: none;display: inline-block;border-radius: 12px;font-size: 16px;' value='Troubleshooting' onclick='msg()'>
                </div><br><br>",
        );

        $possible_search_bases = $ldap_connect->getSearchBases();
        $possible_search_bases_in_key_val = array();
        foreach($possible_search_bases as $search_base){
            $possible_search_bases_in_key_val[$search_base]=$search_base;
        }
        $possible_search_bases_in_key_val['custom_base']='Provide Custom LDAP Search Base';

        $form['miniorange_search_base_options'] = array(
            '#type' => 'value',
            '#id' => 'miniorange_search_base_options',
            '#value' => $possible_search_bases_in_key_val,
        );

      $form['miniorange_ldap_custom_sb_attribute'] = array(
        '#type' => 'textfield',
        '#title' => t('Custom Search Base(s):'),
        '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_custom_sb_attribute'),
        '#states' => array('visible' => array(':input[name = "search_base_attribute"]' => array('value' => 'custom_base' ), ),),
        '#attributes' => array('style' => 'width:73%;'),
      );

        $form['miniorange_search_base_options']['search_base_attribute'] = array(
            '#id' => 'miniorange_ldap_search_base_attribute',
            '#title' => t('Search Base(s):'),
            '#type' => 'select',
            '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_search_base'),
            '#options' => $form['miniorange_search_base_options']['#value'],
            '#attributes' => array('style'=>'width:73%;height:30px'),
        );

      $form['miniorange_ldap_search_base_markup'] = array(
        '#markup'=>  t('This is the LDAP tree under which we will search for the users for authentication. If we are not able to find a user in LDAP it means they are not present in this search base or any of its sub trees.
                Provide the distinguished name of the Search Base object. <b>E.g. cn=Users,dc=domain,dc=com.</b>
               <p> Multiple Search Bases are supported in the <a href="' . $base_url .'/admin/config/people/ldap_auth/Licensing"><strong>Premium, All-inclusive</strong></a> version of the module.</p>'),
      );


        $form['miniorange_username_options'] = array(
            '#type' => 'value',
            '#id' => 'miniorange_username_options',
            '#value' => array(
              'samaccountName' => t('samaccountName'),
              'mail' => t('mail'),
              'userPrincipalName' => t('userPrincipalName'),
              'cn' => t('cn'),
              'custom' => t('Provide Custom LDAP attribute name'),
            ),
          );

          $form['ldap_auth']['settings']['username_attribute'] = array(
            '#id' => 'miniorange_ldap_username_attribute',
            '#title' => t('Search Filter/Username Attribute:'),
            '#type' => 'select',
            '#prefix' => '<br>',
            '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_username_attribute_option'),
            '#options' => $form['miniorange_username_options']['#value'],
            '#attributes' => array('style'=>'width:73%;height:30px'),
          );
          $form['miniorange_ldap_custom_username_attribute'] = array(
            '#type' => 'textfield',
            '#title' => t('Custom Search Filter:'),
            '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_custom_username_attribute'),
            '#states' => array('visible' => array(':input[name = "username_attribute"]' => array('value' => 'custom' ), ),),
            '#attributes' => array('style' => 'width:73%;'),
        );

        $form['miniorange_ldap_username_attribute_options_markup'] = array(
            '#markup' => '<i>Select the LDAP attribute against which the user will be searched. <b>For example:</b> If you want the user to login in Drupal using their email address( the one present in the LDAP server), you can select <b>mail</b> in the dropdown.</i><br>
            <p>You can even search for your user using a Custom Search Filter. You can also allow logging in with multiple attributes, separated with  <i>semicolon</i> <strong>e.g. cn;mail</strong>  <a href="' . $base_url .'/admin/config/people/ldap_auth/Licensing"><strong>[Premium, All-inclusive]</strong></a><br>',
            '#attributes' => array('style'=>'width:73%'),
        );

        $form['save_user_mapping'] = array(
            '#markup' => "
                <div style='background-color: white; padding: 10px ;margin-left: 20px; width: 70%' id='enable_ldap'>
                    <p><b>Please Note:</b> The attributes that we are showing are examples and the actual ones could be different. These should be confirmed with the LDAP Admin.<p>
                    <input type='button' style='background-color: #4CAF50;border: none;color: white;padding: 8px 20px;text-align: center;text-decoration: none;display: inline-block;border-radius: 12px;font-size: 16px;' value='Save User Mapping' onclick='msg()'>
                </div>",
        );
        $form['troubleshooting_2'] = array(
            '#markup' => "
                <div style='background-color: white; padding: 10px ;margin-left: 20px; width: 70%' id='enable_ldap'>
                    <input type='button' style='background-color: #008CBA;border: none;color: white;padding: 8px 20px;text-align: center;text-decoration: none;display: inline-block;border-radius: 12px;font-size: 16px;' value='Troubleshooting' onclick='msg()'>
                </div>",
        );
        $form['back_step_3'] = array(
            '#type' => 'submit',
            '#button_type' => 'danger',
            '#prefix' => '<br><br><div class="inline-align">',
            '#value' => t('Reset Configurations'),
            '#submit' => array('::miniorange_ldap_back_2'),
            '#attributes' => array('style' => 'float:left;'),
        );
        $form['save_config_edit'] = array(
            '#type' => 'submit',
            '#button_type' => 'primary',
            '#suffix' => '</div>',
            '#value' => t('Save Changes'),
            '#submit' => array('::miniorange_ldap_review_changes'),
            '#attributes' => array('style' => 'float: right;'),
        );
        }

    if($status=='one'){
        $form['miniorange_ldap_enable_ldap_markup'] = array(
            '#markup' => "<h1><b>Login With LDAP Options</b></h1><hr><br>",
        );
        $form['miniorange_ldap_enable_ldap'] = array(
            '#type' => 'checkbox',
            '#description' => t('Enabling LDAP login will protect your login page by your configured LDAP. Please check this only after you have successfully tested your configuration as the default Drupal login will stop working'),
            '#title' => t('Enable Login with LDAP '),
            '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_enable_ldap'),
        );
        $form['miniorange_ldap_enable_auto_reg'] = array(
          '#type' => 'checkbox',
          '#title' => t('Enable Auto Registering users if they do not exist in Drupal <a href="' . $base_url .'/admin/config/people/ldap_auth/Licensing"><strong>[Premium, All-inclusive]</strong></a>'),
          '#disabled' => 'true',
          '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_enable_auto_reg'),
        );
        $form['set_of_radiobuttons']['miniorange_ldap_authentication'] = array(
            '#type' => 'radios',
            '#disabled' => 'true',
            '#prefix' =>  '<br>',
            '#suffix' =>  '<br>',
            '#title' => t('Authentication restrictions: <a href="' . $base_url .'/admin/config/people/ldap_auth/Licensing">[Premium, All-inclusive]</a>'),
            '#description' => t('Only particular personalities will be able to login by selecting the above option.'),
            '#tree' => TRUE,
            '#default_value' => is_null(\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_authentication'))?1:\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_authentication'),
          '#options' => array(0 => t('User can login using both their Drupal or LDAP credentials'), 1 => t('User can login in Drupal using their LDAP credentials and Drupal admins can also login using their local Drupal credentials'), 2 => t('Users can only login using their LDAP credentials')),
        );
        $form['back_step_3'] = array(
            '#type' => 'submit',
            '#button_type' => 'danger',
            '#id' => 'button_config',
            '#value' => t('Back'),
            '#submit' => array('::miniorange_ldap_back_5'),
            '#attributes' => array('style' => 'width: fit-content;display:inline-block;'),
        );
        $form['next_step_1'] = array(
            '#type' => 'submit',
            '#button_type' => 'primary',
            '#id' => 'button_config',
            '#value' => t('Save & Review Configurations'),
            '#attributes' => array('style' => 'float: right;display:block;'),
            '#submit' => array('::miniorange_ldap_next_1'),
        );
    }
    else if($status=='two'){
        $form['mo_ldap_local_configuration_form_action'] = array(
            '#markup' => "<input type='hidden' name='option' id='mo_ldap_local_configuration_form_action' value='mo_ldap_local_save_config'></input>",
        );
        $form['ldap_server'] = array(
            '#markup' => "
            <br><h1><b>LDAP Connection Information</b></h1><hr><br><br><h3>LDAP Server:</h3>",
        );

        $form['ldap_server_url_markup_start'] = array(
            '#markup' => '<div class="ldap_Server_row">',
        );

        $form['miniorange_ldap_options'] = array(
            '#type' => 'value',
            '#id' => 'miniorange_ldap_options',
            '#value' => array(
              'ldap://' => t('ldap://'),
              'ldaps://' => t('ldaps://'),),
          );

          $form['miniorange_ldap_protocol'] = array(
            '#id' => 'miniorange_ldap_protocol',
            '#type' => 'select',
            '#prefix' => '<div class="ldap-column left">',
            '#suffix' => '</div>',
            '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_protocol'),
            '#options' => $form['miniorange_ldap_options']['#value'],
            '#attributes' => array('style'=>'width:100%'),
          );

        $form['miniorange_ldap_server_address'] = array(
            '#type' => 'textfield',
            '#id' => 'miniorange_ldap_server_address',
            '#prefix' => '<div class="ldap-column middle">',
            '#suffix' => '</div>',
            '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_server_address'),
            '#attributes' => array('style' => 'width:100%;','placeholder' => 'Enter your server-address or IP'),
        );
        $form['miniorange_ldap_server_port_number'] = array(
            '#type' => 'textfield',
            '#prefix' => '<div class="ldap-column right">',
            '#suffix' => '</div>',
            '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_server_port_number'),
            '#attributes' => array('style' => 'width:100%;','placeholder' => '<port>'),
        );
        $form['ldap_server_url_markup_end'] = array(
            '#markup' => '</div><p><i>Select ldap or ldaps from the above dropdown list. Specify the host name for the LDAP server in the above text field. Edit the port number if you have custom port number.</i></p>',
        );

        $form['miniorange_ldap_contact_server_button'] = array(
            '#type' => 'submit',
            '#value' => t('Contact LDAP Server'),
            '#id' => 'button_config',
            '#submit' => array('::test_ldap_connection'),
        );
        $form['miniorange_ldap_enable_tls'] = array(
            '#prefix' => '<br><br>',
            '#type' => 'checkbox',
            '#id' => 'check',
            '#disabled' => 'true',
            '#title' => t('Enable TLS (Check this only if your server use TLS Connection) <b><a href="' . $base_url .'/admin/config/people/ldap_auth/Licensing"><strong>[Premium, All-inclusive]</strong></a></b>'),
            '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_enable_tls'),
        );
        if(\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_steps') == 1) {
          $form['miniorange_ldap_anonymous_bind_markup'] = array(
            '#markup' => '<br><div class="mo_ldap_highlight_background_note_1" >In case you do not have any authentication setup and wish to perform anonymous bind to your server, please click on the Next button and continue with your setup.</div>',
          );
          $form['miniorange_ldap_server_account_username'] = array(
            '#type' => 'textfield',
            '#title' => t('Service Account Username:'),
            '#required'=>true,
            '#default_value' => \Drupal::config('ldap_auth.settings')->get('miniorange_ldap_server_account_username'),
            '#description' => t("This service account username will be used to establish the connection. Specify the Service Account Username of the LDAP server in the either way as follows Username@domainname or domainname\Username. or Distinguished Name(DN) format"),
            '#attributes' => array('style' => 'width:73%;', 'placeholder' => 'CN=service,DC=domain,DC=com'),
          );
          $form['miniorange_ldap_server_account_password'] = array(
            '#type' => 'password',
            '#title' => t('Service Account Password:'),
            '#required'=>true,
            '#default_value' => \Drupal::config('ldap_auth.settings')->get('miniorange_ldap_server_account_password'),
            '#attributes' => array('style' => 'width:73%;', 'placeholder' => 'Enter password for Service Account'),
          );
          $form['miniorange_ldap_test_connection_button'] = array(
            '#type' => 'submit',
            '#prefix' => '<br>',
            '#value' => t('Test Connection'),
            '#submit' => array('::test_connection_ldap'),
          );
        }
    }
    else if($status=='three'){
      //Get all Search bases from AD
      $possible_search_bases = $ldap_connect->getSearchBases();
      $possible_search_bases_in_key_val = array();
      foreach($possible_search_bases as $search_base){
        $possible_search_bases_in_key_val[$search_base]=$search_base;
      }
      $possible_search_bases_in_key_val['custom_base']='Provide Custom LDAP Search Base';
      $form['miniorange_search_base_options'] = array(
        '#type' => 'value',
        '#id' => 'miniorange_search_base_options',
        '#value' => $possible_search_bases_in_key_val,
      );
      $form['miniorange_search_base_options']['search_base_attribute'] = array(
        '#id' => 'miniorange_ldap_search_base_attribute',
        '#title' => t('Search Base(s):'),
        '#type' => 'select',
        '#description' => t('This is the LDAP tree under which we will search for the users for authentication. If we are not able to find a user in LDAP it means they are not present in this search base or any of its sub trees.
                   Provide the distinguished name of the Search Base object. <b>E.g. cn=Users,dc=domain,dc=com.</b>
                   <p>Multiple Search Bases are supported in the <a href="' . $base_url .'/admin/config/people/ldap_auth/Licensing"><strong>Premium, All-inclusive</strong></a> version of the module.</p>'),
        '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_search_base'),
        '#options' => $form['miniorange_search_base_options']['#value'],
        '#attributes' => array('style'=>'width:73%;height:30px'),
      );
      $form['miniorange_ldap_custom_sb_attribute'] = array(
        '#type' => 'textfield',
        '#title' => t('Custom Search Base(s):'),
        '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_custom_sb_attribute'),
        '#states' => array('visible' => array(':input[name = "search_base_attribute"]' => array('value' => 'custom_base' ), ),),
        '#attributes' => array('style' => 'width:73%;'),
      );
      $form['miniorange_username_options'] = array(
        '#type' => 'value',
        '#id' => 'miniorange_username_options',
        '#value' => array(
          'samaccountName' => t('samaccountName'),
          'mail' => t('mail'),
          'userPrincipalName' => t('userPrincipalName'),
          'cn' => t('cn'),
          'custom' => t('Provide Custom LDAP attribute name'),
        ),
      );
      $form['ldap_auth']['settings']['username_attribute'] = array(
        '#id' => 'miniorange_ldap_username_attribute',
        '#title' => t('Search Filter/Username Attribute:'),
        '#prefix' => '<br>',
        '#type' => 'select',
        '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_username_attribute_option'),
        '#options' => $form['miniorange_username_options']['#value'],
        '#attributes' => array('style'=>'width:73%;height:30px'),
      );
      $form['miniorange_ldap_custom_username_attribute'] = array(
        '#type' => 'textfield',
        '#title' => t('Custom Search Filter:'),
        '#default_value' =>\Drupal::config('ldap_auth.settings')->get('miniorange_ldap_custom_username_attribute'),
        '#states' => array('visible' => array(':input[name = "username_attribute"]' => array('value' => 'custom' ), ),),
        '#attributes' => array('style' => 'width:73%;'),
      );

      $form['miniorange_ldap_username_attribute_options_markup'] = array(
        '#markup' => '<i>Select the LDAP attribute against which the user will be searched. <b>For example:</b> If you want the user to login in Drupal using their email address( the one present in the LDAP server), you can select <b>mail</b> in the dropdown.</i><br>
            <p>You can even search for your user using a Custom Search Filter. You can also allow logging in with multiple attributes, separated with  <i>semicolon</i> <strong>e.g. cn;mail</strong>  <a href="' . $base_url .'/admin/config/people/ldap_auth/Licensing"><strong>[Premium, All-inclusive]</strong></a></p><div><br>',
        '#attributes' => array('style'=>'width:73%'),
      );
      $form['back_step_3'] = array(
        '#type' => 'submit',
        '#button_type' => 'danger',
        '#id' => 'button_config',
        '#prefix' => "<div class='pito_enable_alignment'>",
        '#value' => t('Back'),
        '#submit' => array('::miniorange_ldap_back_3'),
        '#attributes' => array('style' => 'display: inline-block;'),
      );
      $form['next_step_3'] = array(
        '#type' => 'submit',
        '#id' => 'button_config',
        '#button_type' => 'primary',
        '#value' => t('Next'),
        '#suffix' => "</div></div>",
        '#attributes' => array('style' => 'float: right;display:block;'),
        '#submit' => array('::miniorange_ldap_next3'),
      );
    }

   $form['mo_markup_div_imp']=array('#markup'=>'</div>');

    Utilities::AddSupportButton($form, $form_state);
    return $form;
  }
  function miniorange_ldap_back_1($form,$form_state){
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_config_status', 'one')->save();
  }
    function miniorange_ldap_back_2($form,$form_state){
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->clear('miniorange_ldap_enable_ldap')->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->clear('miniorange_ldap_authenticate_admin')->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->clear('miniorange_ldap_authenticate_drupal_users')->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->clear('miniorange_ldap_enable_auto_reg')->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->clear('miniorange_ldap_server')->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->clear('miniorange_ldap_enable_tls')->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->clear('miniorange_ldap_server_account_username')->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->clear('miniorange_ldap_server_account_password')->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->clear('miniorange_ldap_search_base')->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->clear('miniorange_ldap_username_attribute')->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->clear('miniorange_ldap_test_username')->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->clear('miniorange_ldap_test_password')->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_server_port_number', '389')->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_custom_username_attribute', 'samaccountName')->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->clear('miniorange_ldap_server_address')->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->clear('miniorange_ldap_protocol')->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->clear('miniorange_ldap_username_attribute_option')->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_config_status', 'two')->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_steps', "0")->save();
        Utilities::add_message(t('Configurations removed successfully.'),'status');
    }
    function miniorange_ldap_back_3($form,$form_state){
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_config_status', 'two')->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_steps', "1")->save();
    }
    function miniorange_ldap_back_5($form,$form_state){
      \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_steps', "2")->save();
      \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_config_status', 'three')->save();
    }
    function miniorange_ldap_back_4($form,$form_state){
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_config_status', 'four')->save();
    }
    /**
     * Test Connection
     */
    function test_connection_ldap($form,$form_state){
        $server_account_username ="";
        $server_account_password ="";
        $ldap_connect = new LDAPFlow();

        if(isset($_POST['miniorange_ldap_server_account_username']) && !empty($_POST['miniorange_ldap_server_account_username'])){
          $server_account_username = Html::escape(trim($_POST['miniorange_ldap_server_account_username']));
          $ldap_connect->setServiceAccountUsername($server_account_username);
        }
        if(isset($_POST['miniorange_ldap_server_account_password']) && !empty($_POST['miniorange_ldap_server_account_password'])){
            $server_account_password = Html::escape($_POST['miniorange_ldap_server_account_password']);
            $ldap_connect->setServiceAccountPassword($server_account_password);
        }
        user_cookie_save(array("mo_ldap_test" => true));
        $error = array();

        $error = $ldap_connect->test_mo_ldap_config($server_account_username,$server_account_password);

        if( $error[1] == "error" ) {
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_test_connection', 'Tried and Failed')->save();
            if($error[0] == "Invalid Password. Please check your password and try again."){
                \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_server_account_password', '')->save();
                Utilities::add_message(t($error[0]),'error');
            }
            elseif($error[0] == "Username or Password can not be empty until and unless you do not have any authenticaiton setup and wish to perform anonymous bind to your server. If that is the case, please ignore this message and continue with the setup."){
                Utilities::add_message(t($error[0]),'error');
            }
            else
               Utilities::add_message(t("There was an error processing your request."),'error');

        } else if($error[1] == "Success") {
          \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_steps', "2")->save();
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_next_step_base_selection', '1')->save();
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_test_connection', 'Successful')->save();
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_server_account_password', $server_account_password)->save();
            self::miniorange_ldap_next_2($form,$form_state);
             Utilities::add_message(t('Test Connection is successful. Now, select your Search Base, Filter and click on the <b>Next</b> button to continue.'),'status');
        }
    }
    function miniorange_ldap_next_2($form, $form_state){
      $server_address ="";
      $ldap_connect = new LDAPFlow();
      if(isset($_POST['miniorange_ldap_server_address']) && !empty($_POST['miniorange_ldap_server_address']))
        $server_address = Html::escape(trim($_POST['miniorange_ldap_server_address']));
      if(trim($server_address) == ''){
       Utilities::add_message(t('LDAP Server Address can not be empty'),'error');
        return;
      }
      if(isset($_POST['miniorange_ldap_protocol']) && !empty($_POST['miniorange_ldap_protocol']))
        $protocol = Html::escape(trim($_POST['miniorange_ldap_protocol']));
      $server_name = $protocol.$server_address;
      if(isset($_POST['miniorange_ldap_server_port_number']) && !empty($_POST['miniorange_ldap_server_port_number'])){
        $port_number = Html::escape(trim($_POST['miniorange_ldap_server_port_number']));
        $server_name = $server_name.':'.$port_number;
      }

      \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_server_address', $server_address)->save();
      \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_protocol', $protocol)->save();
      \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_server_port_number', $port_number)->save();
      $ldap_connect->setServerName($server_name);
      if(!empty($form['miniorange_ldap_enable_tls']['#value'])){
        $enable_tls = $form['miniorange_ldap_enable_tls']['#value'];
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_enable_tls', $enable_tls)->save();
      }
      if(!empty($form['miniorange_ldap_server_account_username']['#value'])){
        $server_account_username = $form['miniorange_ldap_server_account_username']['#value'];
        $ldap_connect->getServiceAccountUsername($server_account_username);
      }
      if(!empty($form['miniorange_ldap_server_account_password']['#value'])){
        $server_account_password = $form['miniorange_ldap_server_account_password']['#value'];
        $ldap_connect->setServiceAccountPassword($server_account_password);
      }
      \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_config_status', 'three')->save();
    }

    function miniorange_ldap_next3($form, $form_state){
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_config_status', 'one')->save();
        if(!empty($form['miniorange_search_base_options']['search_base_attribute']['#value'])){
            $searchBase = $form['miniorange_search_base_options']['search_base_attribute']['#value'];
            $searchBaseCustomAttribute = null;
            if($searchBase == 'custom_base'){
                \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_username_attribute_option', 'custom')->save();
                $searchBaseCustomAttribute = $form['miniorange_ldap_custom_sb_attribute']['#value'];
            }
            $ldap_connect = new LDAPFlow();
            $ldap_connect->setSearchBase($searchBase,$searchBaseCustomAttribute);
          \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_steps', "3")->save();
        }

        if(!empty($form['ldap_auth']['settings']['username_attribute']['#value'])){
            $usernameAttribute = $form['ldap_auth']['settings']['username_attribute']['#value'];
            $usernameCustomAttribute=null;
            if($usernameAttribute == 'custom'){
                \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_username_attribute_option', 'custom')->save();
                $usernameCustomAttribute = $form['miniorange_ldap_custom_username_attribute']['#value'];
                if(trim($usernameCustomAttribute) == ''){
                    $usernameCustomAttribute = 'samaccountName';
                }
                \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_custom_username_attribute', $usernameCustomAttribute)->save();
                \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_username_attribute', $usernameCustomAttribute)->save();
            }else{
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_username_attribute_option', $usernameAttribute)->save();
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_username_attribute', $usernameAttribute)->save();
            }
        }


        if(!empty($form['miniorange_ldap_test_username']['#value'])){
            $testUsername = $form['miniorange_ldap_test_username']['#value'];
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_test_username', $testUsername)->save();
        }
        if(!empty($form['miniorange_ldap_test_password']['#value'])){
            $testPassword = $form['miniorange_ldap_test_password']['#value'];
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_test_password', $testPassword)->save();
        }
    }
    /**
     * Contact LDAP server
     */
    function test_ldap_connection($form,$form_state){
        global $base_url;
        LDAPLOGGERS::addLogger('L101: Entered Contact LDAP Server ');

        if(!Utilities::isLDAPInstalled()){
            LDAPLOGGERS::addLogger('L102: PHP_LDAP Extension is not enabled');
            Utilities::add_message(t('You have not enabled the PHP LDAP extension'),'error');
            return;
        }

        $server_address ="";
        if(isset($_POST['miniorange_ldap_server_address']) && !empty($_POST['miniorange_ldap_server_address']))
            $server_address = Html::escape(trim($_POST['miniorange_ldap_server_address']));
        if(trim($server_address) == ''){
            Utilities::add_message(t('LDAP Server Address can not be empty.'),'error');
            return;
        }
        if(isset($_POST['miniorange_ldap_protocol']) && !empty($_POST['miniorange_ldap_protocol']))
            $protocol = Html::escape(trim($_POST['miniorange_ldap_protocol']));
        $server_name = $protocol.$server_address;

        if(isset($_POST['miniorange_ldap_server_port_number']) && !empty($_POST['miniorange_ldap_server_port_number'])){
            $port_number = Html::escape(trim($_POST['miniorange_ldap_server_port_number']));
            $server_name = $server_name.':'.$port_number;
        }

      $ldap_connect = new LDAPFlow();
      $ldap_connect->setServerName($server_name);
      $ldapconn = $ldap_connect->getConnection();

        \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_server', $server_name)->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_server_address', $server_address)->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_protocol', $protocol)->save();
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_server_port_number', $port_number)->save();
        if($ldapconn){
          \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_steps', "1")->save();
          \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_contacted_server', "Successful")->save();
          \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_test_conn_enabled', "1")->save();
          Utilities::add_message(t('Congratulations, you were able to successfully connect to your LDAP Server'),'status');
          return;
        }
        else{
          \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_contacted_server', "Failed")->save();
          \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_test_conn_enabled', "0")->save();
          Utilities::add_message(t('There seems to be an error trying to contact your LDAP server. Please check your configurations or contact the administrator for the same.'),'error');
          return;
        }
      }

    function miniorange_ldap_next_1($form, $form_state){
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_config_status', 'review_config')->save();
        $enable_ldap = $form['miniorange_ldap_enable_ldap']['#value'];
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_enable_ldap', $enable_ldap)->save();
        if(!empty($form['miniorange_ldap_authenticate_admin']['#value'])){
            $authn_admin = $form['miniorange_ldap_authenticate_admin']['#value'];
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_authenticate_admin', $authn_admin)->save();
        }
        if(!empty($form['miniorange_ldap_authenticate_drupal']['#value'])){
            $authn_drupal_users = $form['miniorange_ldap_authenticate_drupal']['#value'];
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_authenticate_drupal_users', $authn_drupal_users)->save();
        }
        $auto_reg_users = $form['miniorange_ldap_enable_auto_reg']['#value'];
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_enable_auto_reg', $auto_reg_users)->save();
        user_cookie_save(array("mo_ldap_test" => true));
        $ldap_connect = new LDAPFlow();
        $error = $ldap_connect->test_mo_ldap_config();
      \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_steps', "4")->save();
      if($error[1]=="error")
            Utilities::add_message(t($error[0]),'status');
        else
          Utilities::add_message(t('Configuration updated successfully. <br><br>Now please open a private/incognito window and try to login to your Drupal site using your LDAP credentials. In case you face any issues or if you need any sort of assistance, please feel free to reach out to us at <u><a href="mailto:drupalsupport@xecurify.com"><i>drupalsupport@xecurify.com</i></a></u>'),'status');
        return;
    }

    function miniorange_ldap_review_changes($form, $form_state){
        $enable_ldap = $form['miniorange_ldap_enable_ldap']['#value'];
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_enable_ldap', $enable_ldap)->save();
        if(!empty($form['miniorange_ldap_authenticate_admin']['#value'])){
            $authn_admin = $form['miniorange_ldap_authenticate_admin']['#value'];
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_authenticate_admin', $authn_admin)->save();
        }
        if(!empty($form['miniorange_ldap_authenticate_drupal']['#value'])){
            $authn_drupal_users = $form['miniorange_ldap_authenticate_drupal']['#value'];
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_authenticate_drupal', $authn_drupal_users)->save();
        }
        $auto_reg_users = $form['miniorange_ldap_enable_auto_reg']['#value'];
        \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_enable_auto_reg', $auto_reg_users)->save();
        if(!empty($form['miniorange_ldap_server']['#value'])){
            $mo_ldap_server = $form['miniorange_ldap_server']['#value'];
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_server', $mo_ldap_server)->save();
        }
        if(!empty($form['miniorange_ldap_enable_tls']['#value'])){
            $enable_tls = $form['miniorange_ldap_enable_tls']['#value'];
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_enable_tls', $enable_tls)->save();
        }
        if(!empty($form['miniorange_ldap_server_account_username']['#value'])){
            $server_account_username = $form['miniorange_ldap_server_account_username']['#value'];
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_server_account_username', $server_account_username)->save();
        }
        if(!empty($form['miniorange_ldap_server_account_password']['#value'])){
            $server_account_password = $form['miniorange_ldap_server_account_password']['#value'];
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_server_account_password', $server_account_password)->save();
        }

        if(!empty($form['miniorange_search_base_options']['search_base_attribute']['#value'])){
            $searchBase = $form['miniorange_search_base_options']['search_base_attribute']['#value'];
            if($searchBase == 'custom_base'){
                \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_username_attribute_option', 'custom')->save();
                $searchBaseCustomAttribute = $form['miniorange_ldap_custom_sb_attribute']['#value'];
                \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_custom_sb_attribute', $searchBaseCustomAttribute)->save();
            }
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_search_base', $searchBase)->save();
        }

        if(!empty($form['ldap_auth']['settings']['username_attribute']['#value'])){
            $usernameAttribute = $form['ldap_auth']['settings']['username_attribute']['#value'];
            if($usernameAttribute == 'custom'){
                \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_username_attribute_option', 'custom')->save();
                $usernameCustomAttribute = $form['miniorange_ldap_custom_username_attribute']['#value'];
                if(trim($usernameCustomAttribute) == ''){
                    $usernameCustomAttribute = 'samaccountName';
                }
                \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_custom_username_attribute', $usernameCustomAttribute)->save();
                \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_username_attribute', $usernameCustomAttribute)->save();
            }else{
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_username_attribute_option', $usernameAttribute)->save();
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_username_attribute', $usernameAttribute)->save();
            }
        }
        if(!empty($form['miniorange_ldap_test_username']['#value'])){
            $testUsername = $form['miniorange_ldap_test_username']['#value'];
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_test_username', $testUsername)->save();
        }
        if(!empty($form['miniorange_ldap_test_password']['#value'])){
            $testPassword = $form['miniorange_ldap_test_password']['#value'];
            \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_test_password', $testPassword)->save();
        }
      \Drupal::configFactory()->getEditable('ldap_auth.settings')->set('miniorange_ldap_steps', "4")->save();
      Utilities::add_message(t('Configuration updated successfully. <br><br>Now please open a private/incognito window and try to login to your Drupal site using your LDAP credentials. In case you face any issues or if you need any sort of assistance, please feel free to reach out to us at <u><a href="mailto:drupalsupport@xecurify.com"><i>drupalsupport@xecurify.com</i></a></u>'),'status');
    }

    function submitForm(array &$form, FormStateInterface $form_state){
    }

  function saved_support(array &$form, FormStateInterface $form_state) {
    Utilities::saved_support($form,$form_state);
  }

  function setup_call(array &$form, FormStateInterface $form_state){
    Utilities::setup_call($form,$form_state);
  }
}
